'use strict';
const server = require('http').Server;
const co = require('bluebird').coroutine;
const parser = require('media-typer').parse;
const getRawBody = require('raw-body');
const isStream = require('isstream');
const $ = require('./fn');

const isDev = process.env.NODE_ENV === 'development';
const isTest = process.env.NODE_ENV === 'test';

function send(res, code, obj) {
	if ((obj || null) === null) {
		return res.end();
	}

	res.statusCode = code;

	if (Buffer.isBuffer(obj)) {
		if (!res.getHeader('Content-Type')) {
			res.setHeader('Content-Type', 'application/octet-stream');
		}
		res.setHeader('Content-Length', obj.length);
		return res.end(obj);
	}

	if (isStream(obj)) {
		if (!res.getHeader('Content-Type')) {
			res.setHeader('Content-Type', 'application/octet-stream');
		}
		return obj.pipe(res);
	}

	let str;
	if (typeof obj === 'object') {
		// V8 optimizes if only one argument
		str = isDev ? JSON.stringify(obj, null, 2) : JSON.stringify(obj);
		// stringify first in case of throw 500
		res.setHeader('Content-Type', 'application/json');
	} else {
		str = obj;
	}
	res.setHeader('Content-Length', Buffer.byteLength(str));
	res.end(str);
}

function sendError(req, res, err) {
	const obj = isDev ? err.stack : err.statusCode ? err.message : 'Internal Server Error';
	send(res, err.statusCode || 500, obj);
	if (!isTest) $.error(null, err);
}

function createError(code, msg, orig) {
	const err = new Error(msg);
	err.originalError = orig;
	err.statusCode = code;
	return err;
}

// maps requests to buffered raw bodies so that
// multiple calls to `json` work as expected
const rawBodyMap = new WeakMap();

const json = co(function * (req, opts) {
	opts = opts || {};
	const limit = opts.limit || '1mb';
	const type = req.headers['content-type'];
	const length = req.headers['content-length'];
	const encoding = parser(type).parameters.charset;

	try {
		let str = rawBodyMap.get(req);

		if (!str) {
			str = yield getRawBody(req, {limit, length, encoding});
			rawBodyMap.set(req, str);
		}

		try {
			return JSON.parse(str);
		} catch (err) {
			throw createError(400, 'Invalid JSON', err);
		}
	} catch (err) {
		if (err.type === 'entity.too.large') {
			throw createError(413, `Body exceeded ${limit} limit`, err);
		}
		throw createError(400, 'Invalid body', err);
	}
});

const run = co(function * (req, res, fn) {
	try {
		const val = yield fn(req, res);
		// send no content
		if (val === null) send(res, 204, null);
		// send if not undefined value
		if (val !== undefined) send(res, res.statusCode || 200, val);
	} catch (err) {
		sendError(req, res, err);
	}
});

function serve(fn) {
	if (!$.isGenerator(fn)) {
		$.error('You must supply a GeneratorFunction!');
		process.exit(1);
	}
	fn = co(fn);
	return server((req, res) => run(req, res, fn));
}

module.exports = exports = serve;
exports.createError = createError;
exports.sendError = sendError;
exports.json = json;
exports.send = send;
exports.run = run;
