# polkadot [![Build Status](https://travis-ci.org/lukeed/polkadot.svg?branch=master)](https://travis-ci.org/lukeed/polkadot)

> Async HTTP microservices

Polkadot is **heavily** inspired by [`micro`](https://github.com/zeit/micro). In fact, most of the code is the same!
The only reason `polkadot` exists as a separate module is because it uses [Bluebird](https://github.com/petkaantonov/bluebird) to power its runtime, while `micro` strives for a purely native implementation.

One day (soon?!), Node's native `Promise` will be as fast as Bluebird's version.
Until then, `polkadot` will continue to be [much, **much** faster](https://github.com/lukeed/polkadot/blob/master/benchmarks/readme.md) than `micro`!

## Install

```
$ npm install --save polkadot
```


## Usage

The simplest way to use `polkadot` is through its [CLI](#cli). To do this, simply:

1. Define a `start` script within `package.json`:

  ```js
  {
    "main": "index.js",
    "scripts": {
      "start": "polkadot"
    }
  }
  ```

2. Create an `index.js` file & define the core function:

  ```js
  module.exports = function * (req, res) {
    return 'Hello world!';
  };
  ```

3. Finally, start the server & open `http://localhost:3000` in your browser!

  ```sh
  $ npm start
  ```

### Programmatic

Alternatively, you may require `polkadot` directly for programmatic use.

```js
// server.js
const sleep = require('then-sleep');
const polkadot = require('polkadot');

polkadot(function * (req, res) {
  yield sleep(500);
  return 'Hello!';
}).listen(3000);
```

This server can be initialized by passing the filename to `node`:

```sh
$ node server.js
```

###


## API

### polkadot(fn)
Type: `GeneratorFunction`<br>
Return: `http.Server`

The main function to be run by `polkadot`. Returns a [`http.Server`](https://nodejs.org/dist/latest-v6.x/docs/api/http.html#http_class_http_server) that uses the provided function as the request handler.

See [Programmatic Use](#programmatic) for an example.

### polkadot.send(res, code, data)
See [`micro.send`](https://github.com/zeit/micro#api-1).

### polkadot.json(req, opts)
See [`micro.json`](https://github.com/zeit/micro#api). Can be `yield`ed.

### polkadot.sendError(req, res, error)
See [`micro.sendError`](https://github.com/zeit/micro#api-3).

### polkadot.createError(code, msg, error)
See [`micro.createError`](https://github.com/zeit/micro#api-3).


## CLI

```
$ polkadot --help

  Usage
    $ polkadot [options] <file>

  Options
    -h, --host   Host to listen on [Default: 0.0.0.0]
    -p, --port   Port to listen on [Default: 3000]
        --help   Display this help

  Examples
    $ polkadot server.js
    $ polkadot server.js --port 8080
```

#### file
Default: `index.js`

When using the CLI _without_ defining a `file`, your `package.json` (if exists) will be parsed & its `main` value will be used. See [example](#usage).
If a `package.json` does not exist, or it does not specify a `main` entry, then a root-level `index.js` is assumed.

> **Note:** An error will be thrown if this file also doesn't exist.


## Error Handling

Micro's [error-handling documentation](https://github.com/zeit/micro#error-handling) is great & directly applies to Polkadot. Nothing more to add :)


## From `micro` to `polkadot`

Polkadot stands on `micro`'s successful shoulders.

The code is largely the same, with the exception of _how_ the Promises are formed & handled during runtime. This also means the API is the same!

Migration from `micro` to `polkadot` is very simple:

1. Replace all instances of "micro" with "polkadot"

  ```js
  // old
  const micro = require('micro');
  const { send } = require('micro');
  micro.json(...);

  // new
  const polkadot = require('polkadot');
  const { send } = require('polkadot');
  polkadot.json(...);
  ```

2. Leave all API usage alone: `send`, `json`, `sendError`, `createError`

  ```js
  // old
  micro.json(...);
  micro.send(...);

  // new
  polkadot.json(...);
  polkadot.send(...);
  ```

3. Replace all instances of `await` with `yield`.

  ```js
  const sleep = require('then-sleep');

  // old
  module.exports = async (req, res) => {
    await sleep(500); // <-- here
    return 'Ready!';
  }

  // new
  module.exports = async (req, res) => {
    yield sleep(500); // <-- here
    return 'Ready!';
  }
  ```

4. Replace all instances of `async function` with `function*`.

  ```js
  const sleep = require('then-sleep');

  // old
  module.exports = async (req, res) => { // <-- here
    yield sleep(500);
    return 'Ready!';
  }

  // new
  module.exports = function * (req, res) { // <-- here
    yield sleep(500);
    return 'Ready!';
  }
  ```

## License

MIT © [Luke Edwards](https://lukeed.com)
